@@ -113,6 +113,18 @@ impl EncryptedDeck {
         (deck, key)
     }
 
+    /// returns true if the shuffle_encrypt deck is obviously tampered with, doesn't catch all cases
+    pub fn is_shuffle_encrypt_invalid(&self, num_cards: u32) -> bool {
+        if num_cards != self.cards.len() as u32 {
+            return true;
+        }
+        let num_cards = self.cards.len();
+        let mut cards = self.cards.clone();
+        cards.sort();
+        cards.dedup();
+        num_cards == cards.len()
+    }
+
     /// checks if the shuffle_encrypt mechanism was executed correctly and it can be reversed again
     pub fn is_shuffle_encrypt_valid(&self, key: &Rsa, result: &EncryptedDeck) -> bool {
         let mut decrypted_result = result.decrypt(key);
@@ -150,6 +162,12 @@ impl EncryptedDeck {
         let deck = self.card_specific(shuffle_key, &keys);
         (deck, keys)
     }
+
+    /// returns true if the shuffle_encrypt deck is obviously tampered with, doesn't catch all cases
+    pub fn is_encrypt_card_specific_invalid(&self, num_cards: u32) -> bool {
+        num_cards != self.cards.len() as u32
+    }
+
     /// checks whether the encrypt_card_specific function was executed correctly
     /// needs to know the keys involved, so can only be called for peers after the game has ended
     /// and everyone published all their keys
@@ -164,7 +182,9 @@ impl EncryptedDeck {
 }
 
 impl EncryptedDeck {
-    /// Returns None on error
+    /// Returns None on error. The caller has to check whether there are duplicate cards in the deck
+    /// If there are duplicates the peer who put the first card specific keys on the deck tampered
+    /// with the deck
     ///
     /// # Panics
     ///
@@ -181,7 +201,7 @@ impl EncryptedDeck {
             |card, key| key.decrypt(card),
         );
         let (card, uuid) = Deck::decode_card(decrypted)?;
-        if uuid == shuffle_id {
+        if uuid == shuffle_id && card < self.cards.len() as u32 {
             Some(card)
         } else {
             None
